@@ -1,12 +1,13 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
 #
 import sys
 import os
+import datetime
 import pdb
 import inspect
 from openram import globals
@@ -62,6 +63,9 @@ def print_raw(str):
 
 
 def log(str):
+    # Add timestamp at the beginning of the string
+    timestr = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
+    str = "[{}] {}".format(timestr, str)
     if globals.OPTS.output_name != '':
         if log.create_file:
             # We may have not yet read the config, so we need to ensure
