@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California, Santa Cruz
+# Copyright (c) 2016-2024 Regents of the University of California, Santa Cruz
 # All rights reserved.
 #
 from openram import debug
@@ -16,11 +16,11 @@ from .bitcell_base_array import bitcell_base_array
 class replica_bitcell_array(bitcell_base_array):
     """
     Creates a bitcell array of cols x rows and then adds the replica
-    and dummy columns and rows.  Replica columns are on the left and
+    columns and dummy rows. Replica columns are on the left and
     right, respectively and connected to the given bitcell ports.
-    Dummy are the outside columns/rows with WL and BL tied to gnd.
-    Requires a regular bitcell array, replica bitcell, and dummy
-    bitcell (BL/BR disconnected).
+    Dummy rows are on the top and bottom passing through the RBL WLs.
+    Requires a regular bitcell array and (if using replica topology)
+    replica bitcell and dummy bitcell (BL/BR disconnected).
     """
     def __init__(self, rows, cols, rbl=None, left_rbl=None, right_rbl=None, name=""):
         super().__init__(name=name, rows=rows, cols=cols, column_offset=0)
@@ -36,22 +36,29 @@ class replica_bitcell_array(bitcell_base_array):
         self.column_size = cols
         self.row_size = rows
         # This is how many RBLs are in all the arrays
-        self.rbl = rbl
-        # This specifies which RBL to put on the left or right by port number
-        # This could be an empty list
+        # Even if the RBL is not placed in this array, the module still needs
+        # to place dummy rows with rbl wordlines so that they will have the same
+        # load as the regular wordlines (and so the arrays are the same size)
+        if rbl is not None:
+            self.rbl = rbl
+        else:
+            self.rbl = [0] * len(self.all_ports)
+        # This specifies how many RBLs to put on the left by port number.
+        # For example, left_rbl = [0, 1] means there will be two
+        # RBLs on the left, one for port 0 and another for port 1.
         if left_rbl is not None:
             self.left_rbl = left_rbl
         else:
             self.left_rbl = []
-        # This could be an empty list
+        # Similar to left_rbl but on the right side of the array
         if right_rbl is not None:
             self.right_rbl = right_rbl
         else:
-            self.right_rbl=[]
+            self.right_rbl = []
         self.rbls = self.left_rbl + self.right_rbl
 
         debug.check(sum(self.rbl) >= len(self.left_rbl) + len(self.right_rbl),
-                    "Invalid number of RBLs for port configuration.")
+                    "Cannot have more left + right RBLs than total RBLs")
 
         self.create_netlist()
         if not OPTS.netlist_only:
@@ -64,28 +71,7 @@ class replica_bitcell_array(bitcell_base_array):
         self.create_instances()
 
     def add_modules(self):
-        """  Array and dummy/replica columns
-
-        d or D = dummy cell (caps to distinguish grouping)
-        r or R = replica cell (caps to distinguish grouping)
-        b or B = bitcell
-         replica columns 1
-         v              v
-        bdDDDDDDDDDDDDDDdb <- Dummy row
-        bdDDDDDDDDDDDDDDrb <- Dummy row
-        br--------------rb
-        br|   Array    |rb
-        br| row x col  |rb
-        br--------------rb
-        brDDDDDDDDDDDDDDdb <- Dummy row
-        bdDDDDDDDDDDDDDDdb <- Dummy row
-
-          ^^^^^^^^^^^^^^^
-          dummy rows cols x 1
-
-        ^ dummy columns  ^
-          1 x (rows + 4)
-        """
+        """  Array and dummy/replica columns """
         # Bitcell array
         self.bitcell_array = factory.create(module_type="bitcell_array",
                                             column_offset=1 + len(self.left_rbl),
@@ -96,18 +82,14 @@ class replica_bitcell_array(bitcell_base_array):
         self.replica_columns = {}
 
         for port in self.all_ports:
+            # We will always have self.rbl[0] dummy rows below the array
+            # for the replica wordlines.
             if port in self.left_rbl:
-                # We will always have self.rbl[0] rows of replica wordlines below
-                # the array.
-                # These go from the top (where the bitcell array starts ) down
+                # These go top down starting from the bottom of the bitcell array.
                 replica_bit = self.rbl[0] - port - 1
                 column_offset = len(self.left_rbl)
-
             elif port in self.right_rbl:
-
-                # We will always have self.rbl[0] rows of replica wordlines below
-                # the array.
-                # These go from the bottom up
+                # These go bottom up starting from the top of the bitcell array.
                 replica_bit = self.rbl[0] + self.row_size + port - 1
                 column_offset = len(self.left_rbl) + self.column_size + 1
             else:
@@ -119,11 +101,13 @@ class replica_bitcell_array(bitcell_base_array):
                                                         column_offset=column_offset,
                                                         replica_bit=replica_bit)
 
-        # Dummy row
+        # Dummy row (for replica wordlines)
         self.dummy_row = factory.create(module_type="dummy_array",
                                             cols=self.column_size,
                                             rows=1,
-                                            # dummy column + left replica column
+                                            # cap column + left replica column
+                                            # FIXME: these col offsets should really start at 0 because
+                                            # this is the left edge of the array... but changing them all is work
                                             column_offset=1 + len(self.left_rbl),
                                             mirror=0)
 
@@ -148,7 +132,7 @@ class replica_bitcell_array(bitcell_base_array):
         self.add_pin("gnd", "GROUND")
 
     def add_bitline_pins(self):
-        # The bit is which port the RBL is for
+        # The bit represents which port the RBL is for
         for bit in self.rbls:
             for port in self.all_ports:
                 self.rbl_bitline_names[bit].append("rbl_bl_{0}_{1}".format(port, bit))
@@ -175,6 +159,8 @@ class replica_bitcell_array(bitcell_base_array):
         self.unused_wordline_names = []
 
         for port in self.all_ports:
+            if self.rbl[port] == 0:
+                continue  # TODO: there's probably a better way to do this check
             for bit in self.all_ports:
                 self.rbl_wordline_names[port].append("rbl_wl_{0}_{1}".format(port, bit))
                 if bit != port:
@@ -225,9 +211,12 @@ class replica_bitcell_array(bitcell_base_array):
         self.dummy_row_replica_insts = []
         # Note, this is the number of left and right even if we aren't adding the columns to this bitcell array!
         for port in self.all_ports: # TODO: tie to self.rbl or whatever
-            self.dummy_row_replica_insts.append(self.add_inst(name="dummy_row_{}".format(port),
-                                                                mod=self.dummy_row))
-            self.connect_inst(self.all_bitline_names + self.rbl_wordline_names[port] + self.supplies)
+            if self.rbl[port] != 0:
+                self.dummy_row_replica_insts.append(self.add_inst(name="dummy_row_{}".format(port),
+                                                                    mod=self.dummy_row))
+                self.connect_inst(self.all_bitline_names + self.rbl_wordline_names[port] + self.supplies)
+            else:
+                self.dummy_row_replica_insts.append(None)
 
     def create_layout(self):
 
@@ -249,8 +238,8 @@ class replica_bitcell_array(bitcell_base_array):
         # Array was at (0, 0) but move everything so it is at the lower left
         # We move DOWN the number of left RBL even if we didn't add the column to this bitcell array
         # Note that this doesn't include the row/col cap
-        array_offset = self.bitcell_offset.scale(len(self.left_rbl), self.rbl[0])
-        self.translate_all(array_offset.scale(-1, -1))
+        array_offset = self.bitcell_offset.scale(-len(self.left_rbl), -self.rbl[0])
+        self.translate_all(array_offset)
 
         self.add_layout_pins()
 
@@ -359,7 +348,7 @@ class replica_bitcell_array(bitcell_base_array):
                                         height=self.height)
 
     def route_supplies(self):
-
+        """ just copy supply pins from all instances """
         for inst in self.insts:
             for pin_name in ["vdd", "gnd"]:
                 self.copy_layout_pin(inst, pin_name)
