@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -14,7 +14,6 @@ from openram.base import channel_route
 from openram.base import design
 from openram.base import verilog
 from openram.base import lef
-from openram.router import router_tech
 from openram.sram_factory import factory
 from openram.tech import spice
 from openram import OPTS, print_time
@@ -28,7 +27,6 @@ class sram_1bank(design, verilog, lef):
         design.__init__(self, name)
         lef.__init__(self, ["m1", "m2", "m3", "m4"])
         verilog.__init__(self)
-
         self.sram_config = sram_config
         sram_config.set_local_config(self)
 
@@ -50,6 +48,9 @@ class sram_1bank(design, verilog, lef):
             # Route a M3/M4 grid
             self.supply_stack = self.m3_stack
 
+        # delay control logic does not have RBLs
+        self.has_rbl = OPTS.control_logic != "control_logic_delay"
+
     def add_pins(self):
         """ Add pins for entire SRAM. """
 
@@ -250,18 +251,11 @@ class sram_1bank(design, verilog, lef):
             for inst in self.insts:
                 self.copy_power_pins(inst, pin_name, self.ext_supply[pin_name])
 
-        if not OPTS.route_supplies:
-            # Do not route the power supply (leave as must-connect pins)
-            return
-        elif OPTS.route_supplies == "grid":
-            from openram.router import supply_grid_router as router
-        else:
-            from openram.router import supply_tree_router as router
-        rtr=router(layers=self.supply_stack,
-                   design=self,
-                   bbox=bbox,
-                   pin_type=OPTS.supply_pin_type)
-
+        from openram.router import supply_router as router
+        rtr = router(layers=self.supply_stack,
+                     design=self,
+                     bbox=bbox,
+                     pin_type=OPTS.supply_pin_type)
         rtr.route()
 
         if OPTS.supply_pin_type in ["left", "right", "top", "bottom", "ring"]:
@@ -286,7 +280,7 @@ class sram_1bank(design, verilog, lef):
                                         pin.width(),
                                         pin.height())
 
-        elif OPTS.route_supplies and OPTS.supply_pin_type == "single":
+        elif OPTS.supply_pin_type == "single":
             # Update these as we may have routed outside the region (perimeter pins)
             lowest_coord = self.find_lowest_coords()
 
@@ -324,7 +318,7 @@ class sram_1bank(design, verilog, lef):
             # Grid is left with many top level pins
             pass
 
-    def route_escape_pins(self, bbox):
+    def route_escape_pins(self, bbox=None):
         """
         Add the top-level pins for a single bank SRAM with control.
         """
@@ -368,10 +362,10 @@ class sram_1bank(design, verilog, lef):
                         pins_to_route.append("spare_wen{0}[{1}]".format(port, bit))
 
         from openram.router import signal_escape_router as router
-        rtr=router(layers=self.m3_stack,
-                   design=self,
-                   bbox=bbox)
-        rtr.escape_route(pins_to_route)
+        rtr = router(layers=self.m3_stack,
+                     bbox=bbox,
+                     design=self)
+        rtr.route(pins_to_route)
 
     def compute_bus_sizes(self):
         """ Compute the independent bus widths shared between two and four bank SRAMs """
@@ -517,8 +511,9 @@ class sram_1bank(design, verilog, lef):
         for port in self.read_ports:
             for bit in range(self.word_size + self.num_spare_cols):
                 temp.append("dout{0}[{1}]".format(port, bit))
-        for port in self.all_ports:
-            temp.append("rbl_bl{0}".format(port))
+        if self.has_rbl:
+            for port in self.all_ports:
+                temp.append("rbl_bl{0}".format(port))
         for port in self.write_ports:
             for bit in range(self.word_size + self.num_spare_cols):
                 temp.append("bank_din{0}_{1}".format(port, bit))
@@ -693,7 +688,8 @@ class sram_1bank(design, verilog, lef):
             if port in self.readwrite_ports:
                 temp.append("web{}".format(port))
             temp.append("clk{}".format(port))
-            temp.append("rbl_bl{}".format(port))
+            if self.has_rbl:
+                temp.append("rbl_bl{}".format(port))
 
             # Outputs
             if port in self.read_ports:
@@ -1073,24 +1069,15 @@ class sram_1bank(design, verilog, lef):
         # Some technologies have an isolation
         self.add_dnwell(inflate=2.5)
 
+        init_bbox = self.get_bbox()
         # Route the supplies together and/or to the ring/stripes.
-        # This is done with the original bbox since the escape routes need to
-        # be outside of the ring for OpenLane
-        rt = router_tech(self.supply_stack, 1)
-        init_bbox = self.get_bbox(side="ring",
-                                  margin=rt.track_width)
-
-        # We need the initial bbox for the supply rings later
-        # because the perimeter pins will change the bbox
         # Route the pins to the perimeter
         if OPTS.perimeter_pins:
             # We now route the escape routes far enough out so that they will
             # reach past the power ring or stripes on the sides
-            bbox = self.get_bbox(side="ring",
-                                 margin=11*rt.track_width)
-            self.route_escape_pins(bbox)
-
-        self.route_supplies(init_bbox)
+            self.route_escape_pins(init_bbox)
+        if OPTS.route_supplies:
+            self.route_supplies(init_bbox)
 
 
     def route_dffs(self, add_routes=True):
@@ -1292,18 +1279,19 @@ class sram_1bank(design, verilog, lef):
                 dest_pin = self.bank_inst.get_pin(signal + "{}".format(port))
                 self.connect_vbus(src_pin, dest_pin)
 
-        for port in self.all_ports:
-            # Only input (besides pins) is the replica bitline
-            src_pin = self.control_logic_insts[port].get_pin("rbl_bl")
-            dest_pin = self.bank_inst.get_pin("rbl_bl_{0}_{0}".format(port))
-            self.add_wire(self.m3_stack,
-                          [src_pin.center(), vector(src_pin.cx(), dest_pin.cy()), dest_pin.rc()])
-            self.add_via_stack_center(from_layer=src_pin.layer,
-                                      to_layer="m4",
-                                      offset=src_pin.center())
-            self.add_via_stack_center(from_layer=dest_pin.layer,
-                                      to_layer="m3",
-                                      offset=dest_pin.center())
+        if self.has_rbl:
+            for port in self.all_ports:
+                # Only input (besides pins) is the replica bitline
+                src_pin = self.control_logic_insts[port].get_pin("rbl_bl")
+                dest_pin = self.bank_inst.get_pin("rbl_bl_{0}_{0}".format(port))
+                self.add_wire(self.m3_stack,
+                              [src_pin.center(), vector(src_pin.cx(), dest_pin.cy()), dest_pin.rc()])
+                self.add_via_stack_center(from_layer=src_pin.layer,
+                                          to_layer="m4",
+                                          offset=src_pin.center())
+                self.add_via_stack_center(from_layer=dest_pin.layer,
+                                          to_layer="m3",
+                                          offset=dest_pin.center())
 
     def route_row_addr_dff(self):
         """
