@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -54,14 +54,38 @@ class options(optparse.Values):
     num_spare_cols = 0
 
     ###################
-    # Optimization options
+    # ROM configuration options
+    ###################
+    rom_endian = "little"
+    rom_data = None
+    data_type = "bin"
+    strap_spacing = 8
+    scramble_bits = True
+
+    ###################
+    # Control logic options
     ###################
     # Approximate percentage of delay compared to bitlines
     rbl_delay_percentage = 0.5
 
-    # Allow manual adjustment of the delay chain over automatic
-    auto_delay_chain_sizing = False
+    # FIXME: delay_control_scaling_factor is not used because
+    #        the multi-delay chain is not being sized automatically
+    # if delay chain is automatically sized in delay based control logic
+    # this multiplier can be used to add a guard band to the standard timing
+    # lowering it can improve performance but may cause sram to fail
+    # delay_control_scaling_factor = 1.0
+
+    # multi delay chain is NOT automatically sized, needs to be set by user
+    # list indexes 0 & 1 need to be even for polarity
+    # list indexes 2 - 4 need to be odd for polarity
+    # these default values are the ones used on the September 2023 Chipignite Shuttle
+    # to test delay based control logic with sky130 1rw1r 8x1024 bit (1KB) with 8 column mux
+    multi_delay_chain_pinouts = [2, 10, 11, 17, 31]
+
+    # stages for delay chain in rbl control logic only
     delay_chain_stages = 9
+
+    # fanout per stage for any control logic
     delay_chain_fanout_per_stage = 4
 
     accuracy_requirement = 0.75
@@ -91,6 +115,8 @@ class options(optparse.Values):
     sim_data_path = None
     # A list of load/slew tuples
     use_specified_load_slew = None
+    # Spice simulation raw file
+    spice_raw_file = None
 
     ###################
     # Run-time vs accuracy options.
@@ -100,7 +126,7 @@ class options(optparse.Values):
     # When enabled, layout is not generated (and no DRC or LVS are performed)
     netlist_only = False
     # Whether we should do the final power routing
-    route_supplies = "tree"
+    route_supplies = True
     supply_pin_type = "ring"
     # This determines whether LVS and DRC is checked at all.
     check_lvsdrc = False
@@ -113,14 +139,18 @@ class options(optparse.Values):
     # Output config with all options
     output_extended_config = False
     # Output temporary file used to format HTML page
-    output_datasheet_info = False
+    output_datasheet_info = True
     # Determines which analytical model to use.
     # Available Models: elmore, linear_regression
     model_name = "elmore"
+    # Write graph to a file
+    write_graph = False
 
     ###################
     # Tool options
     ###################
+    # Top process that was ran (openram, memchar, memfunc)
+    top_process = None
     # Use conda to install the default tools
     # (existing tools will be used if disabled)
     use_conda = True
@@ -161,6 +191,15 @@ class options(optparse.Values):
     # Purge the temp directory after a successful
     # run (doesn't purge on errors, anyhow)
 
+    # Bitline, s_en and cell names used in characterizer
+    bl_format = "X{name}{hier_sep}xbank0{hier_sep}bl_{row}_{col}"
+    br_format = "X{name}{hier_sep}xbank0{hier_sep}br_{row}_{col}"
+    sen_format = "X{name}{hier_sep}xbank0{hier_sep}s_en"
+    cell_format = "X{name}{hier_sep}xbank0{hier_sep}xbitcell_array{hier_sep}xreplica_bitcell_array{hier_sep}xbitcell_array{hier_sep}xbit_r{row}_c{col}"
+
+    # Random seed for functional simulation
+    functional_seed = None
+
     # Route the input/output pins to the perimeter
     perimeter_pins = True
 
