@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -67,6 +67,11 @@ class router_tech:
             self.vert_layer_minwidth = max(self.vert_layer_minwidth, max_via_size)
             self.horiz_layer_minwidth = max(self.horiz_layer_minwidth, max_via_size)
 
+            # Update spacing for the new widths
+            max_width = max(self.vert_layer_minwidth, self.horiz_layer_minwidth)
+            self.vert_layer_spacing = self.get_layer_space(1, max_width)
+            self.horiz_layer_spacing = self.get_layer_space(0, max_width)
+
             self.horiz_track_width = self.horiz_layer_minwidth + self.horiz_layer_spacing
             self.vert_track_width = self.vert_layer_minwidth + self.vert_layer_spacing
 
@@ -90,7 +95,7 @@ class router_tech:
         Check if the layers and purposes are the same.
         Ignore if purpose is a None.
         """
-        if lpp1[1] == None or lpp2[1] == None:
+        if lpp1[1] is None or lpp2[1] is None:
             return lpp1[0] == lpp2[0]
 
         return lpp1[0] == lpp2[0] and lpp1[1] == lpp2[1]
@@ -102,30 +107,41 @@ class router_tech:
             return 1
 
     def get_layer(self, zindex):
-        if zindex==1:
+        if zindex == 1:
             return self.vert_layer_name
-        elif zindex==0:
+        elif zindex == 0:
             return self.horiz_layer_name
         else:
             debug.error("Invalid zindex {}".format(zindex), -1)
 
+    def get_lpp(self, zindex):
+        if zindex == 1:
+            return self.vert_lpp
+        elif zindex == 0:
+            return self.horiz_lpp
+        else:
+            debug.error("Invalid zindex {}".format(zindex), -1)
+
     def get_layer_width_space(self, zindex):
         """
         These are the width and spacing of a supply layer given a supply rail
         of the given number of min wire widths.
         """
-        if zindex==1:
-            layer_name = self.vert_layer_name
-        elif zindex==0:
-            layer_name = self.horiz_layer_name
-        else:
-            debug.error("Invalid zindex for track", -1)
-
-        min_wire_width = drc("minwidth_{0}".format(layer_name), 0, math.inf)
-
-        min_width = self.route_track_width * drc("minwidth_{0}".format(layer_name), self.route_track_width * min_wire_width, math.inf)
-        min_spacing = drc(str(layer_name)+"_to_"+str(layer_name), self.route_track_width * min_wire_width, math.inf)
-
+        min_width = self.get_layer_width(zindex)
+        min_spacing = self.get_layer_space(zindex, min_width)
         return (min_width, min_spacing)
 
+    def get_layer_width(self, zindex):
+        """ Return the minimum width of a layer. """
+        layer_name = self.get_layer(zindex)
+        min_wire_width = drc("minwidth_{0}".format(layer_name), 0, math.inf)
+        min_width = self.route_track_width * drc("minwidth_{0}".format(layer_name), self.route_track_width * min_wire_width, math.inf)
+        return min_width
 
+    def get_layer_space(self, zindex, width=None):
+        """ Return the minimum spacing of a layer given wire width. """
+        if width is None:
+            width = self.get_layer_width(zindex)
+        layer_name = self.get_layer(zindex)
+        min_spacing = drc(str(layer_name)+"_to_"+str(layer_name), self.route_track_width * width, math.inf)
+        return min_spacing
