@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California, Santa Cruz
+# Copyright (c) 2016-2024 Regents of the University of California, Santa Cruz
 # All rights reserved.
 #
 import sys, os
@@ -21,7 +21,7 @@ class port_address_test(openram_test):
         openram.init_openram(config_file, is_unit_test=True)
 
         debug.info(1, "Port address 512 rows")
-        a = factory.create("port_address", cols=256, rows=512, port=0)
+        a = factory.create("port_address", cols=256, rows=512, port=0, has_rbl=True)
         self.local_check(a)
 
         openram.end_openram()
