@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California, Santa Cruz
+# Copyright (c) 2016-2024 Regents of the University of California, Santa Cruz
 # All rights reserved.
 #
 import sys, os
@@ -33,9 +33,9 @@ class port_data_1rw_1r_test(openram_test):
         c.words_per_row=4
         c.recompute_sizes()
         debug.info(1, "Four way column mux")
-        a = factory.create("port_data", sram_config=c, port=0)
+        a = factory.create("port_data", sram_config=c, port=0, has_rbl=True)
         self.local_check(a)
-        a = factory.create("port_data", sram_config=c, port=1)
+        a = factory.create("port_data", sram_config=c, port=1, has_rbl=True)
         self.local_check(a)
 
         openram.end_openram()
