@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -31,9 +31,6 @@ class sram_1bank_2mux_global_test(openram_test):
             num_spare_rows = 0
             num_spare_cols = 0
 
-        if OPTS.tech_name == "freepdk45":
-            OPTS.route_supplies = False
-
         c = sram_config(word_size=8,
                         num_words=32,
                         num_banks=1,
