@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -25,6 +25,8 @@ class model_delay_test(openram_test):
         openram.init_openram(config_file, is_unit_test=True)
         OPTS.analytical_delay = False
         OPTS.netlist_only = True
+        OPTS.spice_name = "Xyce"
+        OPTS.num_sim_threads = 8
 
         # This is a hack to reload the characterizer __init__ with the spice version
         from importlib import reload
