@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -188,6 +188,9 @@ def write_drc_script(cell_name, gds_name, extract, final_verification, output_pa
     f.write("expand\n")
     f.write('puts "Finished expanding"\n')
     f.write("drc euclidean on\n")
+    # Workaround to address DRC CIF style not loading if 'drc check' is run before catchup
+    if OPTS.tech_name=="gf180mcu":
+      f.write("drc catchup\n")
     f.write("drc check\n")
     f.write('puts "Finished drc check"\n')
     f.write("drc catchup\n")
