@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2023 Regents of the University of California and The Board
+# Copyright (c) 2016-2024 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -32,6 +32,8 @@ if len(args) != 1:
     print(openram.USAGE)
     sys.exit(2)
 
+# Set top process to openram
+OPTS.top_process = 'openram'
 
 # These depend on arguments, so don't load them until now.
 from openram import debug
@@ -76,4 +78,3 @@ s.save()
 # Delete temp files etc.
 openram.end_openram()
 openram.print_time("End", datetime.datetime.now(), start_time)
-
