@@ -1,6 +1,6 @@
 # See LICENSE for licensing information.
 #
-# Copyright (c) 2016-2022 Regents of the University of California and The Board
+# Copyright (c) 2016-2023 Regents of the University of California and The Board
 # of Regents for the Oklahoma Agricultural and Mechanical College
 # (acting for and on behalf of Oklahoma State University)
 # All rights reserved.
@@ -16,23 +16,14 @@ import os
 TECHNOLOGY = "scn4m_subm"
 
 
-##########################
-# CDK paths
-
-# os.environ["CDK_DIR"] = CDK_DIR #PDK path
-# os.environ["SYSTEM_CDS_LIB_DIR"] = "{0}/cdssetup".format(CDK_DIR)
-# os.environ["CDS_SITE"] = CDK_DIR
-os.environ["MGC_TMPDIR"] = "/tmp"
-
 ###########################
 # OpenRAM Paths
 
-
 try:
     DRCLVS_HOME = os.path.abspath(os.environ.get("DRCLVS_HOME"))
 except:
-    OPENRAM_TECH=os.path.abspath(os.environ.get("OPENRAM_TECH"))
-    DRCLVS_HOME=OPENRAM_TECH+"/scn4m_subm/tech"
+    DRCLVS_HOME = "{0}/tech".format(os.path.dirname(__file__))
+
 os.environ["DRCLVS_HOME"] = DRCLVS_HOME
 
 os.environ["SPICE_MODEL_DIR"] = "{0}/models".format(os.path.dirname(__file__))
